/*******************************************************************\

Module: QBV, The Quantified Boolean Validator

Author: CM Wintersteiger

\*******************************************************************/

#ifndef QBV_H_
#define QBV_H_

#include <map>
#include <vector>
#include <istream>
#include <string>

#include "cnf.h"

class QBV {
  private:
    QCNF qcnf;
    unsigned last_original_clause;
    
    bool  test_trace,
          debug,
          ignore_no_reason;
    
    std::string cur_token;    
    
    char read_character( std::ifstream & );
    std::string& read_token( std::ifstream & );
    unsigned read_unsigned( std::ifstream & );
    signed read_signed( std::ifstream & );
    void read_whitespace( std::ifstream & );
    
    void resolution_line( std::ifstream & );
    void extension_line( std::ifstream & );
    void extension_ite( unsigned, std::ifstream & );
    void extension_and( unsigned, std::ifstream & );
    bool conclusion_line( std::ifstream & );
    
    void trace_line( unsigned, std::ifstream & );
        
    bool testClausewise( std::map<unsigned, signed>& );
    
    // trace checking
    std::vector<unsigned> clause_map;
    
    class Resolution {
      public:
        bool star_resolvent;
        unsigned newinx; 
        DClause resolvent;
        std::list<unsigned> antecedents; 
    };
    std::list<Resolution> work_list;
    void do_resolution( Resolution & );
    
    bool process_trace_line( std::ifstream & );
    void order( const DClause &, 
                std::list<unsigned> &,
                std::list<unsigned> &) const;
  
  public:
  
    unsigned cur_linenumber, cur_colnumber;
    
    QBV( void );
    
    bool validate(const std::string &, const std::string &);
    bool validate(const char *, const char *);
    bool validate_trace(const std::string &);
    bool validate_trace(const char *);
    
    void set_test_trace( void ) { test_trace=true; }
    void set_ignore_no_reason( void ) { ignore_no_reason=true; }
    void set_debug( bool v ) { debug = v; }
    
    
    // QBV::Exception class
    class Exception {
      private:
        std::string msg;
      public:
        Exception(const std::string &emsg) { this->msg = emsg; }
        Exception(const char *e) { this->msg = e; }
        std::string &message( void ) { return msg; }
    };
};

#endif
